import typing as t
import os
from collections import abc
import posixpath

from jinja2 import BaseLoader, TemplateNotFound


class SimpleLoader(BaseLoader):
    """Taken from jinja2s FileSystemLoader and stripped to allow relative and absolut paths.

    Args:
        BaseLoader: BaseClass to inherit get_source fn.
    """

    def __init__(self,
                 searchpath: t.Union[str, os.PathLike, t.Sequence[t.Union[str, os.PathLike]]],
                 encoding: str = "utf-8",
                 ) -> None:
        if not isinstance(searchpath, abc.Iterable) or isinstance(searchpath, str):
            searchpath = [searchpath]

        self.searchpath = [os.fspath(p) for p in searchpath]
        self.encoding = encoding

    def get_source(self, environment, template) -> t.Tuple[str, str, t.Callable[[], bool]]:
        for searchpath in self.searchpath:
            filename = posixpath.join(searchpath, template)
            if not os.path.isfile(filename):
                continue
            f = open(filename, 'rb')
            if f is None:
                continue
            try:
                contents = f.read().decode(self.encoding)
            finally:
                f.close()

            mtime = os.path.getmtime(filename)

            def uptodate() -> bool:
                try:
                    return os.path.getmtime(filename) == mtime
                except OSError:
                    return False

            # Use normpath to convert Windows altsep to sep.
            return contents, os.path.normpath(filename), uptodate
        print("Did not find any templates, looked in the following directories:")
        print([item for item in self.searchpath])
        raise TemplateNotFound(template)
